@@ -17,6 +17,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <curl/curl.h>
+#include <event2/thread.h>
 
 #ifdef HAVE_BREAKPAD_SUPPORT
 #include <c_bpwrapper.h>
@@ -80,9 +81,9 @@ register_rpc_service ()
 
     /* seafile-rpcserver */
     searpc_server_register_function ("seafile-rpcserver",
-                                     seafile_get_session_info,
-                                     "seafile_get_session_info",
-                                     searpc_signature_object__void());
+                                     seafile_sync_error_id_to_str,
+                                     "seafile_sync_error_id_to_str",
+                                     searpc_signature_string__int());
 
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_get_config,
@@ -156,7 +157,7 @@ register_rpc_service ()
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_update_repos_server_host,
                                      "seafile_update_repos_server_host",
-                                     searpc_signature_int__string_string_string());
+                                     searpc_signature_int__string_string());
 
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_disable_auto_sync,
@@ -186,20 +187,16 @@ register_rpc_service ()
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_clone,
                                      "seafile_clone",
-        searpc_signature_string__string_int_string_string_string_string_string_string_string_string_string_string_int_string());
+        searpc_signature_string__string_int_string_string_string_string_string_string_string_int_string());
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_download,
                                      "seafile_download",
-        searpc_signature_string__string_int_string_string_string_string_string_string_string_string_string_string_int_string());
+        searpc_signature_string__string_int_string_string_string_string_string_string_string_int_string());
 
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_cancel_clone_task,
                                      "seafile_cancel_clone_task",
                                      searpc_signature_int__string());
-    searpc_server_register_function ("seafile-rpcserver",
-                                     seafile_remove_clone_task,
-                                     "seafile_remove_clone_task",
-                                     searpc_signature_int__string());
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_get_clone_tasks,
                                      "seafile_get_clone_tasks",
@@ -222,11 +219,6 @@ register_rpc_service ()
                                      "seafile_get_repo_sync_task",
                                      searpc_signature_object__string());
 
-    searpc_server_register_function ("seafile-rpcserver",
-                                     seafile_get_repo_sync_info,
-                                     "seafile_get_repo_sync_info",
-                                     searpc_signature_object__string());
-
     searpc_server_register_function ("seafile-rpcserver",
                                      seafile_find_transfer_task,
                                      "seafile_find_transfer_task",
@@ -475,6 +467,11 @@ main (int argc, char **argv)
     g_thread_init(NULL);
 #endif
 
+#ifndef WIN32
+    /* init multithreading support for libevent.because struct event_base is not thread safe. */
+    evthread_use_pthreads();
+#endif
+
     if (!debug_str)
         debug_str = g_getenv("SEAFILE_DEBUG");
     seafile_debug_set_flags_string (debug_str);
